# --------------------------------------------------------
# Written by Yufei Ye (https://github.com/JudyYe)
# Convert from MATLAB code https://inst.eecs.berkeley.edu/~cs194-26/fa18/hw/proj3/gradient_starter.zip
# --------------------------------------------------------
from __future__ import print_function

import numpy as np
import cv2
import imageio
import matplotlib.pyplot as plt
from skimage.transform import SimilarityTransform, warp

TOY = False
BLEND = True
MIXED = False  # extra credits
COLOR2GRAY = False # extra credits


def toy_recon(image):
    return image


def poisson_blend(fg, mask, bg):
    """
    Poisson Blending.
    :param fg: (H, W, C) source texture / foreground object
    :param mask: (H, W)
    :param bg: (H, W, C) target image / background
    :return: (H, W, C)
    """
    return fg * mask + bg * (1 - mask)


def mixed_blend(fg, mask, bg):
    """EC: Mix gradient of source and target"""
    return fg * mask + bg * (1 - mask)


def color2gray(rgb_image):
    """EC: Convert an RGB image to gray image"""
    return cv2.cvtColor(rgb_image, cv2.COLOR_RGB2GRAY)


def align_source(fg, bg, mask, offset):
    """simple way to warp source to target frame"""
    tform = SimilarityTransform(translation=offset)
    fg = warp(fg, tform, output_shape=bg.shape)
    mask = warp(mask, tform, output_shape=bg.shape)
    return fg, mask


if __name__ == '__main__':
    if TOY:
        image = imageio.imread('toy_problem.png')
        image_hat = toy_recon(image)

        plt.subplot(121)
        plt.imshow(image)
        plt.title('Input')
        plt.subplot(122)
        plt.imshow(image_hat)
        plt.title('Output')
        plt.show()

    if BLEND:
        # after alignment (masking_code.py)
        ratio = 1
        fg = cv2.resize(imageio.imread('source_01.jpg'), (0, 0), fx=ratio, fy=ratio)
        bg = cv2.resize(imageio.imread('target_01.jpg'), (0, 0), fx=ratio, fy=ratio)
        mask = cv2.resize(imageio.imread('mask_01.jpg'), (0, 0), fx=ratio, fy=ratio)

        offset = -np.array([5, 200])
        fg, mask = align_source(fg, bg, mask, offset)

        fg = fg / fg.max()
        bg = bg / bg.max()
        mask = (mask > 0)

        blend_img = poisson_blend(fg, mask, bg)

        plt.subplot(121)
        plt.imshow(fg * mask + bg * (1 - mask))
        plt.title('Naive Blend')
        plt.subplot(122)
        plt.imshow(blend_img)
        plt.title('Poisson Blend')
        plt.show()
    if MIXED:
        # read images
        # ...
        # blend
        img_blend = mixed_blend(fg, mask, bg)
        plt.imshow(blend_img)

    if COLOR2GRAY:
        # read images
        # ...
        gray_image = color2gray(rgb_image)
        plt.imshow(gray_image)

    plt.close()
